import SwiftUI

struct ContentView: View {
    
    var body: some View {
        VStack {
            Spacer()
            
            Text("Hey there, my modifier is BlueBorder()")
                .modifier(BlueBorder())

            Spacer()
            
            Text("Hey there my modifier is CardBackground()")
                .modifier(CardBackground())
            
            Spacer()
            
            Button("Hey there my modifier is CustomButton()") {
                print("Hello, World!")
            }.modifier(CustomButton())
            
            Spacer()
            
            Image(systemName: "rainbow")
                // Make the SF Symbol multi-color
                .renderingMode(.original)
                // Make the image resizable
                .resizable()
                // Resize to 250wx150h
                .frame(width: 250, height: 150)
                // Fun (animate the rainbow)
                .symbolEffect(.variableColor)
                // Use Spoiler()
                .modifier(Spoiler())

            Spacer()
        }
    }
}

struct BlueBorder: ViewModifier {
    func body(content: Content) -> some View {
        return content
            .padding(2)
            .border(.blue, width: 1)
    }
}

struct CardBackground: ViewModifier {
    func body(content: Content) -> some View {
        content
            .padding(10)
            .background(
                Color(UIColor.systemGroupedBackground)
            )
            .cornerRadius(20)
            .shadow(
                color: Color.black.opacity(0.2),
                radius: 4
            )
    }
}

struct CustomButton: ViewModifier {
    func body(content: Content) -> some View {
        content
            .padding(10)
            .background(Color.blue)
            .foregroundColor(.white)
            .cornerRadius(20)
            .shadow(
                color: Color.black.opacity(0.2),
                radius: 4
            )
    }
}

struct Spoiler: ViewModifier {
    @State var isHidden: Bool = true
    
    func body(content: Content) -> some View {
        ZStack(alignment: .center, content: {            
            if isHidden {
                content
                    .layoutPriority(1)
                    .blur(radius: 30)
                    .clipped()
                
                VStack {
                    Image(
                        systemName: "eye.slash.fill"
                    )
                    .foregroundColor(.white)
                    
                    Text("Spoiler")
                        .font(.caption)
                        .bold()
                        .foregroundColor(.white)
                }
            } else {
                content
            }
        })
        .onTapGesture {
            withAnimation { 
                self.isHidden = !self.isHidden
            }            
        }
    }
}
